import datetime
from dataclasses import dataclass

from cardlinky.types.enums.status import Status
from cardlinky.types.enums.currency import Currency


@dataclass
class Payout:
    """
    :param id: str - Unique ID of payout
    :param status: enums.Status - Payout status
    :param amount: float - Payout amount
    :param commission: float - Fees
    :param account_identifier: str - Account to which money will be sent
    :param currency: enums.Currency - Currency
    :param created_at: datetime.datetime - Date and time
    """

    id: str
    status: Status
    amount: float
    commission: float
    account_identifier: str
    currency: Currency
    created_at: datetime.datetime


@dataclass
class PayoutStatus:
    """
    :param id: str - Unique ID of payout
    :param status: enums.Status - Payout status
    :param amount: float - Payout amount
    :param commission: float - Fees
    :param account_identifier: str - Account to which money will be sent
    :param currency: enums.Currency - Currency
    :param created_at: datetime.datetime - Date and time
    :param success: bool - Payout status
    """

    id: str
    status: Status
    amount: float
    commission: float
    account_identifier: str
    currency: Currency
    created_at: datetime.datetime
    success: bool



class Unauthenticated(Exception):
    ...


class MerchantNotFound(Exception):
    ...


class InvalidAmount(Exception):
    ...


class MerchantBanned(Exception):
    ...


class ShopNotFound(Exception):
    ...


class ShopNotEnabled(Exception):
    ...


class AccessDenied(Exception):
    ...


class NoPayout(Exception):
    ...


class GeneralError(Exception):
    ...


class BillNotFound(Exception):
    ...


class MerchantSubscriptionInactive(Exception):
    ...


class MerchantSubscriptionNotFound(Exception):
    ...


class BillIsFinished(Exception):
    ...


class TooManyPayments(Exception):
    ...


class TooManyBills(Exception):
    ...


class PayoutAccountNotFound(Exception):
    ...


class PayoutAccountBanned(Exception):
    ...


class DailyPayoutLimitExceeded(Exception):
    ...


class MonthlyPayoutLimitExceeded(Exception):
    ...


class BalanceNotEnough(Exception):
    ...


class DirectionNotAvailable(Exception):
    ...


class MerchantNotVerified(Exception):
    ...


class PayoutNotFound(Exception):
    ...


class TooManyPayouts(Exception):
    ...


exceptions = {
    "Unauthenticated": Unauthenticated("Invalid API Token"),
    "api:error.merchant_not_found": MerchantNotFound("Merchant is not found in the System"),
    "api:error.invalid_amount": InvalidAmount("Invalid amount"),
    "api:error.merchant_banned": MerchantBanned("Merchant is blocked"),
    "api:error.shop_not_found": ShopNotFound("Shop is not found in the System"),
    "api:error.shop_not_enabled": ShopNotEnabled("Merchant is deactivated"),
    "api:error.access_denied": AccessDenied("Merchant doesn't have access to the shop"),
    "api:error.no_payout": NoPayout("Can't payout in the given currency"),
    "api:error.general_error": GeneralError("Internal error"),
    "api:error.bill_not_found": BillNotFound("Bill doesn't exist"),
    "api:error.merchant_subscription_inactive": MerchantSubscriptionInactive("Subscription is not active"),
    "api:error.merchant_subscription_not_found": MerchantSubscriptionNotFound("Merchant doesn't have a subscription"),
    "api:error.bill_is_finished": BillIsFinished("Bill is paid in case of NORMAL bill"),
    "api:error.too_many_payments": TooManyPayments("You are trying to get too many payments in one request."),
    "api:error.too_many_bills": TooManyBills("You are trying to request too many bills at once."),
    "api:error.payout_account_not_found": PayoutAccountNotFound("Payout account is not found"),
    "api:error.payout_account_banned": PayoutAccountBanned("Payout account is blocked"),
    "api:error.daily_payout_limit_exceeded": DailyPayoutLimitExceeded("Exceeded daily limit"),
    "api:error.monthly_payout_limit_exceeded": MonthlyPayoutLimitExceeded("Exceeded monthly limit"),
    "api:error.balance_not_enough": BalanceNotEnough("Not enough balance for payout"),
    "api:error.direction_not_available": DirectionNotAvailable("Account is unavailable for payout"),
    "api:error.merchant_not_verified": MerchantNotVerified("Merchant doesn't have Verified status"),
    "api:error.payout_not_found": PayoutNotFound("Payout is not found"),
    "api:error.too_many_payouts": TooManyPayouts("You are trying to request to many payouts. Maximum payouts for one request is 1000"),
}



from cardlinky.types.enums.enum import Enum


class Status(Enum):
    NEW: str = "NEW"
    MODERATING: str = "MODERATING"
    PROCESS: str = "PROCESS"
    UNDERPAID: str = "UNDERPAID"
    SUCCESS: str = "SUCCESS"
    OVERPAID: str = "OVERPAID"
    FAIL: str = "FAIL"
    ERROR: str = "ERROR"
    DECLINED: str = "DECLINED"



from dataclasses import dataclass

from cardlinky.types.enums.currency import Currency


@dataclass
class Balance:
    """
    :param currency: Currency - Currency of balance
    :param balance_available: float - Available balance
    :param balance_locked: float - Locked balance for payout
    :param balance_hold: float - Fees
    """

    currency: Currency
    balance_available: float
    balance_locked: float
    balance_hold: float



import datetime
from dataclasses import dataclass

from cardlinky.types.enums.status import Status
from cardlinky.types.enums.currency import Currency
from cardlinky.types.enums.bill_type import BillType


@dataclass
class Bill:
    """
    :param id: str - Unique bill id
    :param status: enums.Status - Bill status
    :param active: boll - Is bill active
    :param amount: float - Bill amount
    :param type: enums.BillType - Bill type. 'Normal' type accepts only one payment.
        'Multi' type accepts unlimited number of payments.
    :param currency_in: enums.Currency - Payment currency
    :param created_at: datetime.datetime - Creation date and time
    """

    id: str
    status: Status
    active: bool
    amount: float
    type: BillType
    currency_in: Currency
    created_at: datetime.datetime


@dataclass
class BillCreate:
    """
    :param success: bool - Payment status
    :param link_url: str - Link to the page with QR-code. Example: https://cardlink.link/link/5QWlqB2kKJ
    :param link_page_url: str - Link to the payment page. Example: https://cardlink.link/transfer/5QWlqB2kKJ
    :param bill_id: str - Unique bill ID
    """

    success: bool
    link_url: str
    link_page_url: str
    bill_id: str


@dataclass
class BillStatus:
    """
    :param id: str - Unique bill id
    :param status: enums.Status - Bill status
    :param active: boll - Is bill active
    :param amount: float - Bill amount
    :param type: enums.BillType - Bill type. 'Normal' type accepts only one payment.
        'Multi' type accepts unlimited number of payments
    :param currency_in: enums.Currency - Payment currency
    :param created_at: datetime.datetime - Creation date and time
    :param created_at: datetime.datetime - Creation date and time
    :param success: bool - Payment status
    """

    id: str
    status: Status
    active: bool
    amount: float
    type: BillType
    currency_in: Currency
    created_at: datetime.datetime
    success: bool


@dataclass
class BillToggleActivity:
    """
    :param id: str - Unique bill id
    :param active: bool - Bill activity flag
    :param status: enums.Status - Bill status
    :param amount: float - Bill amount
    :param type: enums.BillType - Type of bill. NORMAL is for onetime payments and
        MULTI is for infinity number of payments
    :param created_at: datetime.datetime - Bill creation date and time
    :param currency_in: enums.Currency - Currency
    :param success: bool - This flag indicates status of request
    """

    id: str
    active: bool
    status: Status
    amount: float
    type: BillType
    created_at: datetime.datetime
    currency_in: Currency
    success: bool



from __future__ import annotations
import enum
from typing import Optional


class Enum(enum.Enum):
    @classmethod
    def from_value(cls, value: str) -> Optional[Enum]:
        value = value.lower()
        for i in cls:
            if i.value.name.lower() == value:
                return i
        return None



from cardlinky.types.enums.enum import Enum


class BillType(Enum):
    """
    Type of payment link shows how many payments it could receive.
    'normal' type means that only one successful payment could be received for this link.
    'multi' type means that many payments could be received with one link.
    """
    NORMAL: str = "normal"
    MULTI: str = "multi"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cardlinky',
 'cardlinky.types',
 'cardlinky.types.enums',
 'cardlinky.types.models']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.28.2,<3.0.0']

setup_kwargs = {
    'name': 'cardlinky',
    'version': '1.5',
    'description': 'Wrapper for the Cardlink API in Python',
    'long_description': '# cardlinky\n<a href="https://pypi.org/project/cardlinky/"><img src="https://img.shields.io/pypi/v/cardlinky?style=flat-square"></a> <a href="https://pypi.org/project/cardlinky/"><img src="https://img.shields.io/pypi/dm/cardlinky?color=blue&style=flat-square"></a> <img src="https://img.shields.io/pypi/pyversions/cardlinky?style=flat-square"> \n\n[📘 Official documentation](https://cardlink.link/reference/api)\n\n## Usage\nFirstly, you need to create an account and а shop in https://cardlink.link/. After confirmation, you will be able to get a token and a shop ID to work with the API.\n\n### Creating a bill and getting a payment link:\n```py\nfrom cardlinky import Cardlinky\n\n\ndef print_bill_url(token: str, shop_id: str, amount: float) -> None:\n    # Creating an instance of the class\n    cardlinky = Cardlinky(token)\n\n    # Create a bill and save it\n    bill = cardlinky.create_bill(amount=amount, shop_id=shop_id)\n\n    # Getting a payment link and printing\n    print(bill.link_url)\n\n\nprint_bill_url("YOUR-TOKEN", "YOUR-SHOP-ID", 100.0)\n# https://cardlink.link/link/GkLWvKx3\n```\n\n### Getting a bill status:\n```py\nfrom cardlinky import Cardlinky\n\n\ndef print_bill_status(token: str, bill_id: str) -> None:\n    # Creating an instance of the class\n    cardlinky = Cardlinky(token)\n\n    # Create a bill and save it\n    bill_status = cardlinky.get_bill_status(bill_id=bill_id)\n\n    # Getting a status and printing\n    print(bill_status.status)\n\n\nprint_bill_status("YOUR-TOKEN", "BILL-ID")\n# Status.NEW\n```\n\n## Installation\n```sh\npip install cardlinky\n```\n### Dependencies:\nPackage  | Version\n-------- | ----------\n`requests` | `>=2.28.2` \n',
    'author': 'LuK050',
    'author_email': 'volychevk@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/LuK050/cardlinky',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
